package info.bukova.isspst.services.workgroups;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.JobMapping;
import info.bukova.isspst.data.Member;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.data.Workgroup;
import info.bukova.isspst.services.AbstractOwnedService;

import java.util.ArrayList;
import java.util.List;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

public class WorkgroupServiceImpl extends AbstractOwnedService<Workgroup> implements WorkgroupService {
	
	@Override
	public void addMember(Workgroup workgroup, Member member, Role role) {
		if (!member.isPerson() && !role.getAuthority().equals(Constants.ROLE_USER)) {
			throw new WorkgroupException(WorkgroupException.NO_WORGROUP);
		}
		if (workgroup.isCentre() && !role.isCentre()) {
			throw new WorkgroupException(WorkgroupException.MSUT_HAS_CENTER);
		}
		if (!workgroup.isCentre() && !role.isWorkgroup()) {
			throw new WorkgroupException(WorkgroupException.MUST_HAS_WORKGROUP);
		}
		
		if (!checkGroups(workgroup, member)) {
			throw new WorkgroupException(WorkgroupException.CYCLIC_MEMBER);
		}
		
		if (!member.isPerson() && !checkGroups((Workgroup)member, workgroup)) {
			throw new WorkgroupException(WorkgroupException.CYCLIC_MEMBER);
		}
		
		JobMapping mapping = new JobMapping(member, role);
		if (!workgroup.getMembers().contains(mapping)) {
			workgroup.getMembers().add(mapping);
			member.addParent(workgroup);
		}
	}
	
	@Override
	public void removeMember(Workgroup workgroup, JobMapping jobMapping) {
		workgroup.removeMember(jobMapping);
		if (!isMember(workgroup, jobMapping.getMember())) {
			jobMapping.getMember().removeParent(workgroup);
			workgroup.getRemoved().add(jobMapping.getMember());
		}
	}

	@Override
	@Transactional
	public List<Workgroup> getWorkgroups() {
		return this.execQuery("from Workgroup where centre = false");
	}

	@Override
	@Transactional
	public List<Workgroup> getCentres() {
		return this.execQuery("from Workgroup where centre = true");
	}

	@Override
	public boolean isMember(Workgroup workgroup, Member member) {
		for (JobMapping jm : workgroup.getMembers()) {
			if (jm.getMember().equals(member)) {
				return true;
			}
		}
		
		return false;
	}
	
	private boolean checkGroups(Workgroup workgroup, Member member) {
		if (member.equals(workgroup)) {
			return false;
		}
		
		if (member.isPerson()) {
			return true;
		}
		
		Workgroup toAdd = Workgroup.class.cast(member); 
		for (JobMapping jm : toAdd.getMembers()) {
			if (!jm.getMember().isPerson() && !checkGroups(workgroup, jm.getMember())) {
				return false;
			}
		}
		
		return true;
	}

	@Override
	public List<Workgroup> getUserCentres(User user) {
		if (user.getParents() == null) {
			return null;
		}
		List<Workgroup> centres = new ArrayList<Workgroup>();
		for (Workgroup w : user.getParents()) {
			if (w.isCentre()) {
				centres.add(w);
			}
		}
		
		for (Workgroup w : getUserWorkgroups(user)) {
			for (Workgroup centre : w.getParents()) {
				if (!centres.contains(centre)) {
					centres.add(centre);
				}
			}
		}
		return centres;
	}

	@Override
	public List<Workgroup> getUserWorkgroups(User user) {
		if (user.getParents() == null) {
			return null;
		}
		List<Workgroup> wg = new ArrayList<Workgroup>();
		for (Workgroup w: user.getParents()) {
			if (!w.isCentre()) {
				wg.add(w);
			}
		}
		
		return wg;
	}

	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_ADD')")
	public void add(Workgroup entity) {
		super.add(entity);
		updateMembers(entity);
	}

	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_EDIT')")
	public void update(Workgroup entity) {
		super.update(entity);
		updateMembers(entity);
	}
	
	private void updateMembers(Workgroup workgroup) {
		for (JobMapping jm : workgroup.getMembers()) {
			dao.getSession().merge(jm.getMember());			
		}
		
		for (Member m : workgroup.getRemoved()) {
			dao.getSession().merge(m);
		}
	}

	@Override
	public List<Role> getUserWorkgroupRoles(Workgroup workgroup, User user) {
		List<Role> roles = new ArrayList<Role>();
		
		if (!isMember(workgroup, user)) {
			return null;
		}
		
		for (JobMapping jm : workgroup.getMembers()) {
			if (jm.getMember().equals(user)) {
				roles.add(jm.getRole());
			}
		}
				
		return roles;
	}

	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_DELETE')")
	public void delete(Workgroup entity) {
		for (JobMapping m : entity.getMembers()) {
			m.getMember().removeParent(entity);
			dao.getSession().merge(m.getMember());
		}
		super.delete(entity);
	}

}
